const helpers = require('..')
const awsResponseMiddleware = require('../middleware/awsResponseMiddleware')
const caseTransformationMiddleware = require('../middleware/caseTransformationMiddleware')

const {
  compeonHandler,
  log,
  toJsonString,
  toAwsResponse
} = helpers

console.error = jest.fn() // suppress logging when testing

describe('complete example', () => {
  const addNewEventAttr = next => params => next({ event: { ...params.event, newInputKey: 'new input value' } })

  const myHandler = compeonHandler(
    event => ({ body: event.newInputKey }),
    [addNewEventAttr, caseTransformationMiddleware(), awsResponseMiddleware]
  )

  it('applies the middleware correctly', async () => expect(await myHandler()).toMatchSnapshot())
})

const compeonHandlerSnapshot = (title, spec, eventHandler, middleware) => {
  const handler = compeonHandler(eventHandler, middleware)
  describe(title, () => {
    it(spec, async () => {
      const result = await handler({ someKey: 'some value', type: 'stools' })

      expect(result).toMatchSnapshot()
    })
  })
}

describe('compeonHandler', () => {
  compeonHandlerSnapshot('async', 'returns kebab-cased response of handler', async event => event)

  compeonHandlerSnapshot('sync', 'returns kebab-cased response of handler', async event => event)

  compeonHandlerSnapshot('with emtpy handler', 'returns successful response', async () => {})

  compeonHandlerSnapshot(
    'when handler throws Error object',
    'returns json api error',
    async event => { throw new Error('DISAPPOINTED') }
  )

  compeonHandlerSnapshot(
    'when handler throws Error string',
    'returns json api error',
    async event => { throw 'DISAPPOINTED STRING' }
  )

  compeonHandlerSnapshot(
    'when async handler throws Error',
    'returns json api error',
    async event => { throw new Error('DISAPPOINTED') }
  )

  compeonHandlerSnapshot(
    'with middleware',
    'applies middleware to response',
    async event => event,
    [
      caseTransformationMiddleware(),
      awsResponseMiddleware,
      next => async value => {
        const result = await next(value)
        return { ...result, newKey: 'i am in the result' }
      }
    ]
  )

  compeonHandlerSnapshot(
    'with middleware when handler throws error',
    'applies middleware to response',
    async event => { throw 'unangenehm belastend' },
    [
      caseTransformationMiddleware(),
      next => async value => {
        const result = await next(value)
        return { ...result, newKey: 'i am in the result' }
      }
    ]
  )

  describe('when handler throws Error', () => it('logs it', async () => {
    console.error = jest.fn()
    await compeonHandler(event => {
      const error = new Error('this should get logged')
      error.stack = 'mocked stack'
      throw error
    })()

    expect(console.error.mock.calls).toMatchSnapshot()
  }))
})
