const { caseTransformationMiddleware } = require('../../middleware')

describe('caseTransformationMiddleware', () => {
  const event = {
    body: {
      'some-key': {
        'some-other-key': [{ 'key-hey': 'test' }]
      }
    },
    'key-outside-of-body': 'test'
  }
  const response = {
    body: {
      camelcaseKey: [{ keyA: 'test' }]
    },
    keyOutsideOfBody: 'test'
  }

  describe('with request and response transformation', () => {
    const nextMock = jest.fn(() => response)
    const middleware = caseTransformationMiddleware()(nextMock)

    it('transforms the request and response', async () => {
      const result = middleware({ event })

      expect(nextMock.mock.calls[0]).toMatchSnapshot()
      expect(await result).toMatchSnapshot()
    })
  })

  describe('when only the request shall be transformed', () => {
    const nextMock = jest.fn(() => response)
    const middleware = caseTransformationMiddleware({ transformResponse: false })(nextMock)

    it('transforms only the request', async () => {
      const result = middleware({ event })

      expect(nextMock.mock.calls[0]).toMatchSnapshot()
      expect(await result).toMatchSnapshot()
    })
  })

  describe('when only the response shall be transformed', () => {
    const nextMock = jest.fn(() => response)
    const middleware = caseTransformationMiddleware({ transformRequest: false })(nextMock)

    it('transforms only the response', async () => {
      const result = middleware({ event })

      expect(nextMock.mock.calls[0]).toMatchSnapshot()
      expect(await result).toMatchSnapshot()
    })
  })

  describe('when the request body is missing', () => {
    const event = {}
    const nextMock = jest.fn(() => response)
    const middleware = caseTransformationMiddleware()(nextMock)

    it('passes on the event', async () => {
      const result = middleware({ event })

      expect(nextMock.mock.calls[0]).toMatchSnapshot()
      expect(await result).toMatchSnapshot()
    })
  })

  describe('when the response body is missing', () => {
    const response = {}
    const nextMock = jest.fn(() => response)
    const middleware = caseTransformationMiddleware()(nextMock)

    it('passes on the response', async () => {
      const result = middleware({ event })

      expect(nextMock.mock.calls[0]).toMatchSnapshot()
      expect(await result).toMatchSnapshot()
    })
  })
})
