const {
  handleAxiosError,
  handleAxiosOk,
  handleEmptyOk,
  handleError,
  handleOk,
  handleRedirect
} = require('../../utils/responses')

describe('handleOk', () => {
  describe('without an argument', () => it('returns 200 AWS response', () => {
    expect(handleOk()).toEqual({ statusCode: 200 })
  }))

  describe('with argument', () => it('returns 200 AWS response with body', () => {
    expect(handleOk('with an argument')).toEqual({ body: 'with an argument', statusCode: 200 })
  }))
})

describe('handleError', () => {
  it('logs to console.error', () => {
    console.error = jest.fn()
    handleError('unangenehm belastend')

    expect(console.error.mock.calls).toMatchSnapshot()
  })

  it('returns AWS response with JSON API error body', () => {
    const result = handleError('unangenehm belastend')
    const jsonApiError = { errors: [{
      code: 'Internal Server Error',
      detail: 'unangenehm belastend',
      status: 500,
      title: '500 (Internal Server Error)'
    }] }

    expect(result).toEqual({ body: jsonApiError, statusCode: 500 })
  })
})

describe('handleEmptyOk', () => it('behaves like handleOk without arguments', () => {
  const result = handleEmptyOk()

  expect(result).toEqual(handleOk())
  expect(result).toEqual({ statusCode: 200 })
}))

describe('handleAxiosOk', () => it('extracts `data` and passes it to handleOk', () => {
  const axiosResponse = { data: { info: 'Cool' }, other: 'stuff' }

  expect(handleAxiosOk(axiosResponse)).toEqual(handleOk(axiosResponse.data))
}))

describe('handleAxiosError', () => it('extracts `response` and passes it to handleError', () => {
  const [data, status] = ['Belastung', 404]
  const axiosResponse = { response: { data, status }, other: 'stuff' }

  expect(handleAxiosError(axiosResponse)).toEqual(handleError(data, status))
}))

describe('handleRedirect', () => {
  it('returns default 302 AWS response with Location header', () => {
    expect(handleRedirect('example.com')).toMatchSnapshot()
  })

  it('returns custom status AWS response with Location header', () => {
    expect(handleRedirect('example.com', 304)).toMatchSnapshot()
  })
})
