const { buildCloudFrontHeaders, deepKebabTransform } = require('../../utils/transformers')

describe('deepKebabTransform', () => {
  it('transforms keys deeply', () => {
    const result = deepKebabTransform({
      type: 'tests',
      someAttribute: 'some attribute',
      nestedObject: {
        nestedAttribute: 'juten tach',
        anotherNestedObject: {
          soMuchNesting: 'please stop'
        }
      },
      someArray: [
        {
          objectInsideOfArray: 'so hard',
          muchChallenge: {
            nestedObjInsdeArray: 'very accepted'
          }
        },
        'unexpected string!',
        [
          { arrayInArray: 'value' }
        ]
      ]
    })

    expect(result).toMatchSnapshot()
  })

  it('transforms arrays', () => {
    expect(deepKebabTransform(['an', 'array'])).toEqual(['an', 'array'])
    expect(deepKebabTransform([
      [
        { arrayInArray: 'val' },
        { myKey: 'val', nested: { nestedKey: 'val' } }
      ],
      [
        { anotherArrayInArray: 'val' }
      ]
    ])).toMatchSnapshot()
  })

  it('equals identity if parameter is not an Object or Array', () => {
    expect(deepKebabTransform('a string')).toEqual('a string')
  })
})

describe('buildCloudFrontHeaders', () => {
  it('transforms headers for use in edge lambdas', () => {
    const result = {
      itLowercasesKeyOnOutside: buildCloudFrontHeaders({
        'sOmE-wIld-CaPitAliZaTiOn': 'and a value'
      }),
      itLeavesTheValueUntouched: buildCloudFrontHeaders({
        'X-My-Key': { some: 'value to be transformed later' }
      }),
    }

    expect(result).toMatchSnapshot()
  })
})
