const transform = require('lodash.transform')

const DEFAULT_EVENT_CONFIG = Object.freeze({
  cloudFront: false
})

class Request {
  get isOptions () {
    return this.method === 'OPTIONS'
  }

  header (name) {
    return this.headers[name.toLowerCase()]
  }
}

class LambdaRequest extends Request {
  constructor (event) {
    super()
    this.request = event
  }

  get headers () {
    this._headers = this._headers || transform(this.request.headers, (result, value, key) => {
      result[key.toLowerCase()] = value
    })

    return this._headers
  }

  get method () {
    return this.request.httpMethod.toUpperCase()
  }
}

class CloudFrontRequest extends Request {
  constructor (event) {
    super()
    this._event = event
    this.request = event.Records[0].cf.request
  }

  get method () {
    return this.request.method.toUpperCase()
  }

  get headers () {
    this._headers = this._headers || transform(this.request.headers, (result, value, key) => {
      result[key] = value[0].value
    })

    return this._headers
  }
}

const requestFromEvent = (event, options) => {
  const { cloudFront } = { ...DEFAULT_EVENT_CONFIG, ...options }
  return cloudFront ? new CloudFrontRequest(event) : new LambdaRequest(event)
}

module.exports = {
  CloudFrontRequest,
  LambdaRequest,
  requestFromEvent
}
