const { jsonApiErrorTransform } = require('./transformers')
const { toJsonString } = require('./logging')

const handleAxiosError = ({ response }) => handleError(response.data, response.status)

const handleAxiosOk = ({ data }) => handleOk(data)

const handleError = (error, status = 500) => {
  const statusCode = error.status || status
  const message = typeof error === 'string' ? error : error.message
  const stack = error.stack

  console.error(toJsonString({ statusCode, message, stack }))

  return { body: jsonApiErrorTransform({ message, status: statusCode }), statusCode }
}

// to avoid the `body` parameter when chaining in promises
const handleEmptyOk = () => handleOk()

const handleOk = (body, headers, statusCode = 200, cloudFront = false) => {
  const result = cloudFront ? { status: statusCode } : { statusCode }
  if (body) result.body = body
  if (headers) result.headers = headers

  return result
}

const handleRedirect = (url, statusCode = 302) => ({ statusCode, headers: { Location: url } })

module.exports = {
  handleAxiosError,
  handleAxiosOk,
  handleEmptyOk,
  handleError,
  handleOk,
  handleRedirect
}
