const statusMap = require('statuses')

const kebabCase = require('lodash.kebabcase')
const camelCase = require('lodash.camelcase')

const isObject = obj => obj instanceof Object && obj.constructor === Object

const buildCloudFrontHeaders = (headers) => {
  const result = {}

  for (const key in headers) {
    result[key.toLowerCase()] = [{ key, value: headers[key] }]
  }

  return result
}

const deepKeyTransformer = transformFunc => {
  const transform = data => {
    if (Array.isArray(data)) return data.map(transform)
    if (!isObject(data)) return data

    let result = {}

    Object.keys(data).forEach(key => result[transformFunc(key)] = transform(data[key]))

    return result
  }

  return transform
}

const jsonApiErrorTransform = (error) => {
  const detail = error.message
  const status = error.status || 500// integer
  const code = statusMap[status] // string
  const title = `${status} (${code})`

  return { errors: [{ status, code, title, detail}] }
}

module.exports = {
  buildCloudFrontHeaders,
  deepKeyTransformer,
  deepCamelTransform: deepKeyTransformer(camelCase),
  deepKebabTransform: deepKeyTransformer(kebabCase),
  jsonApiErrorTransform
}
